function findMinPhotonThreshold(start_DataDir, k, ImSZ, Indx, Channels, ...
                                ChannelLabel, smf, Verbose)
% Starting with the value in smf for Thresholding_MinPhotons for channel k,
% perform a fit using this threshold and produce a corresponding histogram of
% the photon intensities of the localizations detected for a series of selected
% joined image files.  This process may need to be iterated a few times.  When
% a final value is decided upon, change the corresponding value in the
% Initialization section of SiMPullMain as well.
%
% INPUTS:
%    start_DataDir   start location to look for data files
%    k               channel number
%    ImSZ            image linear dimension, typically 256 [pixels]
%    Indx            x and y quadrant indices for DIPimage format
%    Channels        channel numbers that are being used
%    ChannelLabel    cell array of strings identifying each channel
%    smf             Single Molecule Fitting parameters for each channel
%    Verbose         if > 0, produce additional output

   [FileName, FilePath] = uigetfile(fullfile(start_DataDir, 'joined_*.mat'),...
                                    'Select joined images to fit',          ...
                                    'MultiSelect','on');
   % Just make sure FileName is a cell (in case only one file was selected).
   if ~iscell(FileName)
      FileName = {FileName};
   end

   % Strip the extensions out of the file names for ease of later comparison.
   for i = 1 : numel(FileName)
      [~, FileName{i}, ~] = fileparts(FileName{i});
   end

   SaveDir = fullfile(FilePath, 'Figures');
   if ~isfolder(SaveDir)
      mkdir(SaveDir);
   end

   if Verbose > 0
      fprintf('MinPhotons = %d\n', smf.Thresholding_MinPhotons(k));
   end

   % fileinfo.m MUST be in the same directory as the joined files, so that
   % bad frames can be properly excluded.
   run(fullfile(FilePath, 'fileinfo.m'));

   % Copied from overlayFitting.
   for iiDat=1:length(dat)
      % Initialize good frames (it doesn't consider first and last frame
      % (0 and 4) because they are used for focusing).
      goodFrames{iiDat}= 1:3;
      if exist('badFrames', 'var')
         goodFrames{iiDat}([badFrames{iiDat}])= []; % Remove bad frames
      end
      for iiFrame=1:length(goodFrames{iiDat})
         % create structure with frames to be analyzed
         dat(iiDat).frms{iiFrame}=goodFrames{iiDat}(iiFrame);
      end
   end
   filenames = { dat.filename };

   PSFSigmaBlob = 1.3;
   Photons = [];
   for i = 1 : numel(FileName)
      % File should contain 'datasetJoined'.
      load(fullfile(FilePath, FileName{i}));
      % process image, crop and save
      % variable for the current frames to analyze
      % Find the current filename in the list from fileinfo.  If a match is
      % found, use the index j to extract the indices of the good frames.
      TF = strcmp(FileName{i}, filenames);
      if any(TF)
         j = find(TF);
         currfrms = cell2mat(dat(j).frms);
      else
         currfrms=1:3;
      end
      fullsequence=squeeze(datasetJoined(:,:,:));
      % do a sum projection of all frames
      test_Chk=sum(fullsequence(:,:,currfrms),[],3);
      test_Chk=squeeze(test_Chk);

      % create sequence of 5 frames- just repeated image so that SPT code can
      % be used for fitting
      sequence = repmat(test_Chk(Indx{k}{1}, Indx{k}{2}), [1 1 5]);

      % Do not plot boxes irrespective of the Verbose level.
      [TrackXY, tots, overlayIm, shiftIm, SMD] =                    ...
         SiMPullUtils.fitblob(k, ImSZ, Channels, ChannelLabel, smf, ...
                              PSFSigmaBlob, 0, sequence);
      Photons = [Photons; SMD.Photons];

      % ---------- plotBox ----------

      % This code is repeated to simply produce the box plots for the first
      % good frame of each file.

      % do a sum projection of the first good frame of the file
      test_Chk=sum(fullsequence(:,:,currfrms(1)),[],3);
      test_Chk=squeeze(test_Chk);

      % create sequence of 5 frames- just repeated image so that SPT code can
      % be used for fitting
      sequence = repmat(test_Chk(Indx{k}{1}, Indx{k}{2}), [1 1 5]);
      % Instead, plot boxes only for the first good frame of each file.
      [TrackXY, tots, overlayIm, shiftIm, SMD] =                    ...
         SiMPullUtils.fitblob(k, ImSZ, Channels, ChannelLabel, smf, ...
                              PSFSigmaBlob, Verbose, sequence);
      set(gcf, 'Name', sprintf('Boxes: Dataset %d, ROI %d, Channel %d', ...
                               i, currfrms(1), k));
   end

   figure;
   axes('FontSize', 15, 'FontWeight', 'bold');
   hold on
   histogram(Photons, 50);
   xlabel('Photons per localization');
   ylabel('Count');
   title(sprintf('Thresholding.MinPhotons = %d', ...
         smf.Thresholding_MinPhotons(k)));
   hold off
   if Verbose > 0
      SaveFile = sprintf('Hist_%s_MinPhotons=%', ...
                         ChannelLabel{k}, smf.Thresholding_MinPhotons(k));
      saveas(gcf, fullfile(SaveDir, SaveFile), 'png');
   end

   fprintf('Done findMinPhotonThreshold.\n');

end
