%findOverlaps finds overlaps between 2 and possibly 3 coordinate arrays.
function [nOverlaps12, tots12, nOverlaps123, tots123, coord123] = ...
   findOverlaps(ImSZ, pixallow, PSFSigmaBlob, track1, track2, track3)
%
% INPUTS:
%    ImSZ          image linear dimension, typically 256 [pixels]
%    pixallow      number of pixels allowed to be shifted between overlapping
%                  blobs to consider them as being the same molecule
%    PSFSigmaBlob  PSF sigma in the Gaussian blob images [pixels]
%    track1        (m x 2) coordinate array [pixels]
%    track2        (n x 2) coordinate array [pixels]
%    track3        (p x 2) coordinate array [pixels] [OPTIONAL]
%
% OUTPUTS:
%   nOverlaps12    number of overlapping blobs found between 1 and 2
%   tots12         sum blob image exhibiting overlaps found between 1 and 2
%   nOverlaps123   number of overlapping blobs found between 1, 2 and 3
%   tots123        sum blob image exhibiting overlaps found between 1, 2 and 3
%   coord123       (q x 2) triple overlap coordinate array

% Created by:
%    Michael Wester (2020)

   % Blank image used in the process of building gaussian blob images.
   blnk = dip_image(zeros(ImSZ, ImSZ));
   nOverlaps12 = 0; % number of overlapping blobs

   % pdist2 finds the distances between all fits in track1 relative to track2.
   % Distance here means center-to-center distance of fits.
   % track1/2/3 are n x 2 coordinate arrays of single image localizations where
   % 1/2/3 corresponds to different channels (colors).
   % find(D < pixallow) finds those distances less than pixallow, set in the
   % calling program, and assigns the fit indices to ii (track1) and jj
   % (track2), producing a correspondence between the fits in the two tracks.
   % NOTE: The overlap coordinates, here and below in the triple overlap
   % section, are currently taken to be the coordinates of the channel 1 fits.
   D = pdist2(track1, track2);
   %fprintf('[D12 min, D12 max] = [%f, %f]\n', min(min(D)), max(max(D)));
   [ii, jj] = find(D < pixallow);
   nOverlaps12 = numel(ii);
   % tots12 (and tots123 below) are constructed simply for display purposes.
   if nOverlaps12 > 0
      SZ = [ImSZ, ImSZ, 1];
      SMD.X = track1(ii, 1);
      SMD.Y = track1(ii, 2);
      SMD.X_SE = 0.25 * ones(size(SMD.X));
      SMD.Y_SE = 0.25 * ones(size(SMD.X));
      SMD.Photons = ones(size(SMD.X)) * 5000;
      SMD.FrameNum = ones(size(SMD.X));
      SMD.Bg = 0;
      SMD.PSFSigma = PSFSigmaBlob;
      SMD.Z = [];
      SMD.XSize = SZ(1);
      SMD.YSize = SZ(2);
      SMD.NDatasets = 1;
      SMD.NFrames = SZ(3);
      %tots12 = gaussianblob(blnk, track1(ii, :), 1, 5000, 'spatial', 3);
      %tots12 = dip_image(SMA_Sim.gaussBlobImage(SZ(1:2), SZ(3), SMD));
      tots12 = dip_image(smi_sim.GaussBlobs.gaussBlobImage(SMD));
   else
      tots12 = blnk;
   end
   % Create a 3D rather than a 2D image for compatibility.
   tots12 = reshape(tots12, [size(tots12), 1]);

   if ~exist('track3', 'var')
      return;
   end

   % Now, take the overlapping fits from channels 1 and 2 (track12), and use
   % them to find any overlaps with channel 3 (track3).  ij and kk are the
   % respective fit indices for the overlaps.  The overlap coordinates are
   % saved in coord123.
   track12 = track1(ii, :);
   D = pdist2(track12, track3);
   %fprintf('[D12 min, D12 max] = [%f, %f]\n', min(min(D)), max(max(D)));
   [ij, kk] = find(D < pixallow);
   nOverlaps123 = numel(ij);
   if nOverlaps123 > 0
      SZ = [ImSZ, ImSZ, 1];
      SMD.X = track1(ii, 1);
      SMD.Y = track1(ii, 2);
      SMD.X_SE = 0.25 * ones(size(SMD.X));
      SMD.Y_SE = 0.25 * ones(size(SMD.X));
      SMD.Photons = ones(size(SMD.X)) * 5000;
      SMD.FrameNum = ones(size(SMD.X));
      SMD.Bg = 0;
      SMD.PSFSigma = PSFSigmaBlob;
      SMD.Z = [];
      SMD.XSize = SZ(1);
      SMD.YSize = SZ(2);
      SMD.NDatasets = 1;
      SMD.NFrames = SZ(3);
      tots123 = dipshow(smi_sim.GaussBlobs.gaussBlobImage(SMD));
   else
      tots123 = blnk;
   end
   % Create a 3D rather than a 2D image for compatibility.
   tots123 = reshape(tots123, [size(tots123), 1]);
   coord123 = track12(ij, :);

end
