% fitblob generates localizations from the raw image data provided in sequence,
% shifting the data (when k = channel number > 1) based on the previously
% computed GeomTransform.
function [TrackXY, tots, overlayIm, shiftIm, SMDthresh] =           ...
   fitblob(k, ImSZ, Channels, ChannelLabel, smf, Verbose, sequence, ...
           GeomTransform)
%
% INPUTS:
%    k               channel number
%    ImSZ            image linear dimension, typically 256 [pixels]
%    Channels        channel numbers that are being used
%    ChannelLabel    cell array of strings identifying each channel
%    smf             data structure containing parameters for fitStack
%    Verbose         if true, display the fit box plot
%    sequence        image to process (N x N x 1 pixels)
%    GeomTransform   [OPTIONAL] geometric transform (k > 1) to apply to the fit
%                    coordinates
%
% OUTPUTS:
%    TrackXY         shifted coordinates of the fit found
%    tots            Gaussian blob image of the shifted coordinates (k > 1)
%    overlayIm       overlay of the unshifted Gaussian blob image produced from
%                    the fit with the scaled original sequence image
%    shiftIm         overlay of the unshifted Gaussian blobs with the shifted
%                    ones
%    SMDthresh       SMD structure generated from the image after thresholding

% Created by:
%    Michael Wester (2020)

   if ~ismember(k, Channels)
      blnk = reshape(dip_image(zeros(ImSZ, ImSZ)), ImSZ, ImSZ, 1);
      TrackXY = single(zeros(0, 2));
      tots = blnk;
      overlayIm = blnk;
      shiftIm = blnk;
      SMDthresh = [];
      return;
   end

   % Use SPT to find positions of channel k.
   chlab = ChannelLabel{k};
   SMF = smi_core.SingleMoleculeFitting();
   SMF.BoxFinding.BoxOverlap   = smf.BoxFinding_BoxOverlap(k);
   SMF.BoxFinding.BoxSize      = smf.BoxFinding_BoxSize(k);
   SMF.BoxFinding.MinPhotons   = smf.BoxFinding_MinPhotons(k);
   SMF.Data.CameraGain         = smf.Data_CameraGain(k);
   SMF.Data.CameraOffset       = smf.Data_CameraOffset(k);
   SMF.Fitting.FitType         = smf.Fitting_FitType{k};
   SMF.Fitting.PSFSigma        = smf.Fitting_PSFSigma(k);
   SMF.Thresholding.MaxXY_SE   = smf.Thresholding_MaxXY_SE(k);
   SMF.Thresholding.MinPhotons = smf.Thresholding_MinPhotons(k);
   SMF.Thresholding.MinPValue  = smf.Thresholding_MinPValue(k);

   % Generate localizations from the raw data contained in sequence.
   fprintf('   smi_core.DataToPhotons(sequence_%s)\n', chlab);
   % fitStack produces an overlay image (as its 4th argument), but the model
   % is green and the data is red which is hard to see, so below create an
   % overlay image in which the model (Model) is red and the data
   % (ScaledData) is green.
   % Earlier in SiMPull, sequences of 5 identical frames are saved, which
   % causes fits to be repeated 5 times unless just one frame is examined.
   DTP = smi_core.DataToPhotons(SMF, im2mat(sequence(:, :, 1)));
    DTP.Verbose = 0;
   ScaledData = DTP.convertData();
   LDVerbose = 0;
   AutoRun = 1;   % run genLocalizations() in the constructor
   [~, SMD, SMDPreThresh] = ...
      smi_core.LocalizeData(ScaledData, SMF, LDVerbose, AutoRun);
   SMDthresh = SMD;
   if Verbose > 0
      smi_core.FindROI.plotBox(SMDPreThresh, ScaledData, 1, ...
                               SMF.BoxFinding.BoxSize);
   end
   %print(gcf, '-r300', sprintf('Boxes_%s', chlab), '-dpng');
   fprintf('   Done smi_core.DataToPhotons(sequence_%s)\n', chlab);

   % Recreate Model (as done in fitStack), here corresponding to uniformly
   % sized spots at a high intensity.  This will replace the gaussianblob
   % image that was used originally as gaussianblob and gaussBlobImage
   % use different conventions for blob centering (gaussianblob uses the
   % coordinate input as the blob center while gaussBlobImage uses the upper
   % left corner of the pixel).
   SZ = size(ScaledData);
   SMD.X_SE = 0.25 * ones(size(SMD.Photons));
   SMD.Y_SE = 0.25 * ones(size(SMD.Photons));
   SMD.Photons = ones(size(SMD.Photons)) * 5000;
   SMD.Z = [];
   SMD.XSize = SZ(1);
   SMD.YSize = SZ(2);
   SMD.NDatasets = 1;
   SMD.NFrames = size(ScaledData, 3);
   Model = smi_sim.GaussBlobs.gaussBlobImage(SMD);

   % Note that SMD.X/Y are unshifted coordinates and X/Y, TrackXY are
   % shifted.
   if k == 1
      % The first channel has unshifted coordinates since it is the
      % reference.
      TrackXY = [SMD.X, SMD.Y];  % must be Nx2 matrix
      fprintf('   -> size(TrackXY_%s) = %d\n', chlab, size(TrackXY, 1));
   else
      % Shift tracks from the kth channel (as suggested by David Schodt).
      [Xtemp, Ytemp] = transformPointsInverse(GeomTransform, SMD.X, SMD.Y);
      X = 2*SMD.X - Xtemp;
      Y = 2*SMD.Y - Ytemp;
      TrackXY = [X, Y]; % must be Nx2 matrix
      fprintf('   -> size(TrackXY_%s) = %d (shifted)\n', ...
              chlab, size(TrackXY, 1));
      fprintf('   %s: mean SMD.X -> X = %f px\n', ...
              chlab, mean(X - SMD.X));
      fprintf('   %s: mean SMD.Y -> Y = %f px\n', ...
              chlab, mean(Y - SMD.Y));
   end
   % Ensure that TrackXY has two columns for subsequent operations.
   if isempty(TrackXY)
      TrackXY = single(zeros(0, 2));
   end
   SMD_shifted = SMD;
   SMD_shifted.X = TrackXY(:, 1);
   SMD_shifted.Y = TrackXY(:, 2);
   SMD_shifted.Z = [];
   SMD_shifted.XSize = SZ(1);
   SMD_shifted.YSize = SZ(2);
   SMD_shifted.NDatasets = 1;
   SMD_shifted.NFrames = size(ScaledData, 3);
   % tots is the Gaussian blob image of the shifted coordinates (k > 1).
   tots = smi_sim.GaussBlobs.gaussBlobImage(SMD_shifted);
   % Create a 3D rather than a 2D image for compatibility.
   tots = reshape(dip_image(tots), [size(tots), 1]);

   % Produce the sum image of the sequence.  Note that ScaledData and Model are
   % unshifted as they come directly from the sequence.
   if numel(size(ScaledData)) == 2
      % ScaledData/Model dimensions are N x N.  Typically, N = 256.
      sumScaledData = dip_image(ScaledData);
      unshifted_tots = dip_image(Model);
      sumScaledData = reshape(sumScaledData, [size(sumScaledData), 1]);
      unshifted_tots = reshape(unshifted_tots, [size(unshifted_tots), 1]);
   else
      % ScaledData/Model dimensions are N x N x f (f = number of frames).
      sumScaledData = sum(dip_image(ScaledData), [], 3);
      unshifted_tots = sum(dip_image(Model), [], 3);
   end
   % Create an overlay of the unshifted Gaussian blob image produced from the
   % fit with the scaled original sequence image.
   % The max(1, below is to prevent division by zero.
   scaling = max(max(max(sumScaledData))) / max(1, max(max(unshifted_tots)));
   overlayIm = joinchannels('RGB', unshifted_tots * scaling, sumScaledData);

   % Produce an overlay of the unshifted Gaussian blobs with the shifted ones.
   shiftIm = joinchannels('RGB', unshifted_tots, tots);

end
