function overlayFitting(start_DataDir, ImSZ, Indx, Channels, ChannelLabel, ...
                        pixallow, smf, Verbose)
% Fit the localizations in each channel (using fitblob), then overlay all
% possible channel pairs (and 3 at once) to find overlaps (findOverlaps),
% saving the results (results.mat and results.xls) for the summaryPlots.

% INPUTS:
%    start_DataDir     start location to look for data files
%    ImSZ              linear image size (pixel)
%    Indx              x and y quadrant indices for DIPimage format---see
%                      FiducialChannel definitions below
%    Channels          channel numbers that are being used
%    ChannelLabel      cell array of strings identifying each channel
%    pixallow          number of pixels allowed to be shifted between
%                      overlapping blobs to consider them as being the same
%                      molecule
%    smf               Single Molecule Fitting parameters for each channel
%    Verbose           if > 0, produce additional output (e.g., fit box plots)

% ---------- Overlay Fitting Prep ----------

% define files
filedir = uigetdir(start_DataDir, 'Directory containing files to be tracked');

% Create folder to save files for tracking
savedir=fullfile(filedir,'totrack');
if ~exist(savedir, 'dir')
  mkdir(savedir);
end

% Create folder to save figures
savedirFigs=fullfile(filedir,'Figures');
if ~exist(savedirFigs, 'dir')
  mkdir(savedirFigs);
end

% Defined location and name of Registration Analysis files
%regAnal_2 =fullfile(filedir, 'fiducial', 'GeomTransform_R.mat');
%regAnal_3 =fullfile(filedir, 'fiducial', 'GeomTransform_FR.mat');
regAnal_2 =fullfile(filedir, 'fiducial', ...
                    ['GeomTransform_', ChannelLabel{2}, '.mat']);
regAnal_3 =fullfile(filedir, 'fiducial', ...
                    ['GeomTransform_', ChannelLabel{3}, '.mat']);

% ##### BEGIN Set file info #####

% IMPORTANT: to make your life easier, use formatFilenames where you can
% select the files and they will get formatted automatically.
%
% CHANGE OF TERMINOLOGY: we replace "movie" by "dataset" below, and in output
% will change "frame" to "ROI", but to simplify life, will retain "frame" in
% comments and variable names.
%
% For all datasets to be processed:
%
% Put the filename and badFrames into a file called 'fileinfo.m' in filedir
% to not further clutter up this current script.  badFrames can be omitted if
% all the frames in each file are fine.  For example,
%
% dat(1).filename='nameWithoutExtension';
% badFrames{1} = [1 3];
% % Define bad frames (start counting with 0 as DIPimage); leave empty or omit
% % entirely if no bad frames.
%
% Remember: Location in 4D image is indicated in the top of the window:
% allDatasets(x,y), where x indicates the number of the field of view or
% frame, and y the number of the file.

run(fullfile(filedir, 'fileinfo.m'));

% ##### END Set file info #####

for iiDat=1:length(dat)
    % Initialize good frames (it doesn't consider first and last frame (0 and
    % 4) because they are used for focusing).
    goodFrames{iiDat}= 1:3;
    if exist('badFrames', 'var')
       goodFrames{iiDat}([badFrames{iiDat}])= []; % Remove bad frames
    end
    for iiFrame=1:length(goodFrames{iiDat})
        % create structure with frames to be analyzed
        dat(iiDat).frms{iiFrame}=goodFrames{iiDat}(iiFrame);
    end
end

% Load Registration Analysis files and get transformation functions
GeomTransform_2 = [];
GeomTransform_3 = [];
if ismember(2, Channels)
   load(regAnal_2);
   GeomTransform_2 = ParamStruct.ChannelRegInfo.GeomTransform;
end
if ismember(3, Channels)
   load(regAnal_3);
   GeomTransform_3 = ParamStruct.ChannelRegInfo.GeomTransform;
end

totFrms= 0; % Initialize total number of frames to be analyzed
% Calculate total number of frames to be analyzed
for ii=1:size(dat,2)
    totFrms= totFrms+size(dat(ii).frms(:),1);
end

% Before looping through, initiate column identifiers and 'valuecell'
% to store the numbers of found fits

% define column identifiers
colIdent= {'FileName', '488-fits','561-fits','642-fits',...
'488-561-overlaps','488-642-overlaps','561-642-overlaps','Triple-overlap',...
'% 488 w/561','Average %','% 488 w/642','Average %','% 488 w/561-642','Average %',...
'% 488-561 w/642','Average %','% 488-642 w/561','Average %'};

valuecell= cell(totFrms+1,numel(colIdent));
valuecell(1,:)= colIdent;
% Absolute frame number (to correctly place info in cell array 'valuecell')
% Blank image used in the process of building gaussian blob images
absFrameNum= 0;
blnk=dip_image(zeros(ImSZ, ImSZ));

fprintf('Done overlay prep.\n');

% ---------- Overlay Fitting ----------

% loop through each dataset defined above
for ii=1:size(dat,2)
     load(fullfile(filedir,dat(ii).filename)); 
     % because sequence var gets re-written need to rename, Now
     % I'm using dataset instead of sequence so this renaming may
     % be unnecesary in the future
    for jj=1:length(dat(ii).frms)
    fprintf('### Dataset %d/%d ROI %d/%d ...\n', ...
            ii, size(dat, 2), jj, length(dat(ii).frms));
        clear sptObj642 sptObj488 test currfrms image_out c svfilename TrackXY642 TrackXY488 tots488 tots642

        % Absolute frame number (to correctly place info in cell array
        % 'valuecell')
        absFrameNum= absFrameNum+1;

        % process image, crop and save
        currfrms=dat(ii).frms{jj}; % variable for the current frames to analyze
        fullsequence=squeeze(datasetJoined(:,:,:)); 
        % do a sum projection of all frames
        test_488=sum(fullsequence(:,:,currfrms),[],3);
        test_488=squeeze(test_488);

        fullsequence=squeeze(datasetJoined(:,:,:)); 
        % do a sum projection of all frames
        test_561=sum(fullsequence(:,:,currfrms),[],3);
        test_561=squeeze(test_561);

        fullsequence=squeeze(datasetJoined(:,:,:));
        % do a sum projection of all frames
        test_642=sum(fullsequence(:,:,currfrms),[],3);
        test_642=squeeze(test_642);

        svfilename=[dat(ii).filename '_' mat2str(currfrms(1)-1) '-' mat2str(currfrms(size(currfrms,2)))]; % create filestring with filename and frame range
        
        % set default width of window to 512 to display next image properly
        dipsetpref('DefaultFigureWidth', 512,'DefaultFigureHeight', 512);
        h_raw=dipshow(stretch(test_642)); % use to show the image
        set(h_raw, 'Name', ...
                   sprintf('Raw: Dataset %d, ROI %d', ii, jj));
        saveas(h_raw,fullfile(savedirFigs,[svfilename '_raw.png']),'png')
        saveas(h_raw,fullfile(savedirFigs,[svfilename '_raw.fig']),'fig')

        % create sequence of 5 frames- just repeated image so that SPT code can
        % be used for fitting
        % save 488 of image
        %sequence488=repmat(test_488(256:end,0:255),[1 1 5]);
        sequence488=repmat(test_488(Indx{1}{1}, Indx{1}{2}), [1 1 5]);
        sequence = sequence488;
        save(fullfile(savedir,                                            ...
                      sprintf('%s_%s.mat', svfilename, ChannelLabel{1})), ...
             'sequence');
        
        % save 561 of image
        %sequence561=repmat(test_561(0:255,0:255),[1 1 5]);
        sequence561=repmat(test_561(Indx{2}{1}, Indx{2}{2}), [1 1 5]);
        sequence = sequence561;
        save(fullfile(savedir,                                            ...
                      sprintf('%s_%s.mat', svfilename, ChannelLabel{2})), ...
             'sequence');
        
        % save 642 of image
        %sequence642=repmat(test_642(0:255,256:511),[1 1 5]);
        sequence642=repmat(test_642(Indx{3}{1}, Indx{3}{2}), [1 1 5]);
        sequence = sequence642;
        save(fullfile(savedir,                                            ...
                      sprintf('%s_%s.mat', svfilename, ChannelLabel{3})), ...
             'sequence');

        % --- 488 -------------------------------------------------------------

        % PSF sigma for fitblob Gaussian blob images (pixel)
        PSFSigmaBlob = 1.3;

        [TrackXY488, tots488, overlayIm488] =                         ...
           SiMPullUtils.fitblob(1, ImSZ, Channels, ChannelLabel, smf, ...
                                PSFSigmaBlob, Verbose, sequence488);
        %set(gcf, 'NumberTitle', 'off', 'Name', ...
        set(gcf, 'Name', ...
                 sprintf('Boxes: Dataset %d, ROI %d, Channel %d', ii, jj, 1));
        if Verbose > 0
           saveas(gcf, fullfile(savedirFigs,                          ...
              sprintf('%s_%s_F%d_boxes', svfilename, ChannelLabel{1}, ...
                                         currfrms)), 'png');
           close
        end
        %dipshow(overlayIm488, 'lin');
        %diptruesize(gcf, 135);
        [~, overlay488] = SiMPullUtils.diptomat(overlayIm488, 'GM');
        saveas(gcf, fullfile(savedirFigs,                           ...
                             sprintf('%s_%s_overlayed',             ...
                                     svfilename, ChannelLabel{1})), ...
                    'png');
        close
        % Save overlay image as a .mat file.
        save(fullfile(savedirFigs,                           ...
                      sprintf('%s_%s_overlayed',             ...
                              svfilename, ChannelLabel{1})), ...
             'overlay488');

        % --- 561 -------------------------------------------------------------

        [ShiftedTrackXY561, tots561, overlayIm561, shiftIm561] =      ...
           SiMPullUtils.fitblob(2, ImSZ, Channels, ChannelLabel, smf, ...
                                PSFSigmaBlob, Verbose, sequence561,   ...
                                GeomTransform_2);
        set(gcf, 'Name', ...
                 sprintf('Boxes: Dataset %d, ROI %d, Channel %d', ii, jj, 2));
        if Verbose > 0
           saveas(gcf, fullfile(savedirFigs,                          ...
              sprintf('%s_%s_F%d_boxes', svfilename, ChannelLabel{2}, ...
                                         currfrms)), 'png');
           close
        end
        %dipshow(overlayIm561, 'lin');
        %diptruesize(gcf, 135);
        [~, overlay561] = SiMPullUtils.diptomat(overlayIm561, 'GM');
        saveas(gcf, fullfile(savedirFigs,                           ...
                             sprintf('%s_%s_overlayed',             ...
                                     svfilename, ChannelLabel{2})), ...
                    'png');
        close
        % Save overlay image as a .mat file.
        save(fullfile(savedirFigs,                           ...
                      sprintf('%s_%s_overlayed',             ...
                              svfilename, ChannelLabel{2})), ...
             'overlay561');
        %save shifted values
        save(fullfile(savedir, sprintf('%s_%s_shiftedvals.mat',       ...
                                       svfilename, ChannelLabel{2})), ...
             'ShiftedTrackXY561');

        % --- 642 --------------------------------------------------------------

        [ShiftedTrackXY642, tots642, overlayIm642, shiftIm642] =      ...
           SiMPullUtils.fitblob(3, ImSZ, Channels, ChannelLabel, smf, ...
                                PSFSigmaBlob, Verbose, sequence642,   ...
                                GeomTransform_3);
        set(gcf, 'Name', ...
                 sprintf('Boxes: Dataset %d, ROI %d, Channel %d', ii, jj, 3));
        if Verbose > 0
           saveas(gcf, fullfile(savedirFigs,                          ...
              sprintf('%s_%s_F%d_boxes', svfilename, ChannelLabel{3}, ...
                                         currfrms)), 'png');
           close
        end
        %dipshow(overlayIm642, 'lin');
        %diptruesize(gcf, 135);
        [~, overlay642] = SiMPullUtils.diptomat(overlayIm642, 'GM');
        saveas(gcf, fullfile(savedirFigs,                           ...
                             sprintf('%s_%s_overlayed',             ...
                                     svfilename, ChannelLabel{3})), ...
                    'png');
        close
        % Save overlay image as a .mat file.
        save(fullfile(savedirFigs,                           ...
                      sprintf('%s_%s_overlayed',             ...
                              svfilename, ChannelLabel{3})), ...
             'overlay642');
        %save shifted values
        save(fullfile(savedir, sprintf('%s_%s_shiftedvals.mat',       ...
                                       svfilename, ChannelLabel{3})), ...
             'ShiftedTrackXY642');

        % ---------------------------------------------------------------------

        fprintf('size of 488, 561, 642 = %d, %d, %d\n', size(TrackXY488, 1),...
                size(ShiftedTrackXY561, 1), size(ShiftedTrackXY642, 1));
        % overlay shifted blobs on raw data
        ovlayimage=joinchannels('rgb',tots642,tots488,tots561);
        %h_raw=dipshow(ovlayimage, 'lin'); % use to show the image
        %diptruesize(gcf, 135);
        h_raw = SiMPullUtils.diptomat(ovlayimage);
        set(gcf, 'Name', ...
                 sprintf('Shifted Overlay: Dataset %d, ROI %d', ii, jj));

        % save shifted images
        saveas(h_raw,fullfile(savedirFigs,[svfilename '_overlayedchannels.fig']),'fig')
        saveas(h_raw,fullfile(savedirFigs,[svfilename '_overlayedchannels.png']),'png')
%        close(h)% close image after saving

% --- SQUARE

        % --- 561 overlapping 488 ---------------------------------------------

        % PSF sigma for findOverlaps Gaussian blob images (px)
        PSFSigmaBlob = 1.3;

        % Calculate number of spots in 561 channel overlapping with spots in
        % 488 channel 
        [numoverlap561w488, tots561ov488] = ...
           SiMPullUtils.findOverlaps(ImSZ, pixallow, PSFSigmaBlob, ...
                                     TrackXY488, ShiftedTrackXY561);
        fprintf('numoverlap561w488 = %d\n', numoverlap561w488);

        % --- 642 overlapping 488 ---------------------------------------------

        % Calculate number of spots in 642 channel overlapping with spots in
        % 488 channel 
        [numoverlap642w488, tots642ov488] = ...
           SiMPullUtils.findOverlaps(ImSZ, pixallow, PSFSigmaBlob, ...
                                     TrackXY488, ShiftedTrackXY642);
        fprintf('numoverlap642w488 = %d\n', numoverlap642w488);

        % --- 642 overlapping 561 ---------------------------------------------

        % Calculate number of spots in 642 channel overlapping with spots in
        % 561 channel 
        [numoverlap642w561, tots642ov561, numTripleOverlap, ...
         totsTripleOverlap, coordTripleOv] = ...
           SiMPullUtils.findOverlaps(ImSZ, pixallow, PSFSigmaBlob,  ...
                                     TrackXY488, ShiftedTrackXY561, ...
                                                 ShiftedTrackXY642);
        fprintf('numoverlap642w561 = %d\n', numoverlap642w561);
        fprintf('numTripleOverlap  = %d\n', numTripleOverlap);

        % --- triple overlap --------------------------------------------------

        % Calculate number of spots overlapping in the three channels
        % (488, 561 and 642)

        % Remove rows with all zeros and save coordinates
        coordTripleOv(all(coordTripleOv==0,2),:)=[];
        save(fullfile(savedir,                                ...
                      sprintf('%s_%s_tripleOverlapCoord.mat', ...
                              svfilename, ChannelLabel{1})),  ...
             'coordTripleOv');

        % ---------------------------------------------------------------------

        % Display image with only of those spots overlapping with the ones
        % in 488 channel
        if isempty(tots642ov488)==0 || isempty(tots561ov488)==0
        % shows all green blobs and only those red (642) and blue (561) blobs
        % that overlay
        ovlayOnly=joinchannels('rgb',tots642ov488,tots488,tots561ov488);
        %h_ovlayOnly=dipshow(ovlayOnly, 'lin'); % use to show the image
        %diptruesize(gcf, 135);
        h_ovlayOnly = SiMPullUtils.diptomat(ovlayOnly);
        set(gcf, 'Name', ...
                 sprintf('Overlay Only on Channel 1: Dataset %d, ROI %d', ...
                         ii, jj));
        saveas(h_ovlayOnly, fullfile(savedirFigs,                           ...
                                     sprintf('%s_overlayW%sonly.png',       ...
                                             svfilename, ChannelLabel{1})), ...
              'png')
% --- SQUARE
        else
        end

        % For reference below are the current column identifiers for the
        % cell array or spread sheet
%       colIdent= ...
%  1      {'FileName', ...
%  2       '488-fits', ...
%  3       '561-fits', ...
%  4       '642-fits', ...
%  5       '488-561-overlaps', ...
%  6       '488-642-overlaps', ...
%  7       '561-642-overlaps', ...
%  8       'Triple-overlap', ...
%  9       '% 488 w/561', ...
% 10       'Average %', ...
% 11       '% 488 w/642', ...
% 12       'Average %', ...
% 13       '% 488 w/561-642', ...
% 14       'Average %', ...
% 15       '% 488-561 w/642', ...
% 16       'Average %', ...
% 17       '% 488-642 w/561', ...
% 18       'Average %'};

        % Concatenate results to cell array (see column identifiers above)
        % Add 1 (+1) to absFrameNum to count for column identifiers
        valuecell(absFrameNum+1,:)= ...
          {svfilename, ...
           size(TrackXY488,1), ...
           size(ShiftedTrackXY561,1), ...
           size(ShiftedTrackXY642,1), ...
           numoverlap561w488, ...
           numoverlap642w488, ...
           numoverlap642w561, ...
           numTripleOverlap,...
           numoverlap561w488/size(TrackXY488,1)*100, ...
           '', ...
           numoverlap642w488/size(TrackXY488,1)*100, ...
           '', ...
           numTripleOverlap/size(TrackXY488,1)*100, ...
           '', ...
           numTripleOverlap/numoverlap561w488*100, ...
           '', ...
           numTripleOverlap/numoverlap642w488*100, ...
           ''}; 

        % Create overlayed localizations and raw data
        locMask=newim(2*ImSZ,2*ImSZ); % Create figure to save mask with localizations
        % Add mask to corresponding areas/channel
        maxval = 100;
        %locMask(256:end,   0:255) = squeeze(tots488 > maxval);
        %locMask(  0:255,   0:255) = squeeze(tots561 > maxval);
        %locMask(  0:255, 256:511) = squeeze(tots642 > maxval);
        locMask(Indx{1}{1}, Indx{1}{2}) = squeeze(tots488 > maxval);
        locMask(Indx{2}{1}, Indx{2}{2}) = squeeze(tots561 > maxval);
        locMask(Indx{3}{1}, Indx{3}{2}) = squeeze(tots642 > maxval);

        h_rawOverlay=dipshow(overlay(stretch(test_642),locMask>0)); % use to show the image
        set(h_rawOverlay, 'Name', ...
                          sprintf('Raw Overlay: Dataset %d, ROI %d', ii, jj));
        saveas(h_rawOverlay,fullfile(savedirFigs,[svfilename '_rawOverlayLoc.fig']),'fig')
        saveas(h_rawOverlay,fullfile(savedirFigs,[svfilename '_rawOverlayLoc.png']),'png')
% --- SQUARE

    end
    %close all
end

%after loop write to excel and matlab files
% data saved in matlab file
save(fullfile(filedir, 'results.mat'),'valuecell','coordTripleOv')
% data saved in excel file
xlswrite(fullfile(filedir, 'results.xls'),valuecell)

fprintf('Done overlay fitting.\n');

end
